

class Constants:

    @staticmethod
    def raise_large_exception():

        raise Exception("""
                        Lorem ipsum dolor sit amet, consectetur adipiscing elit. Donec malesuada mauris in lectus 
                        congue interdum. Praesent vitae diam est. Donec ullamcorper a dui ac facilisis. Aliquam a quam 
                        quis nisi pharetra mollis ac id velit. Suspendisse lobortis arcu metus, nec posuere nisi 
                        euismod sit amet. Nunc vel tristique sapien, ut convallis dolor. Aliquam non metus vestibulum, 
                        finibus sem a, suscipit felis. Donec volutpat a turpis at varius. Donec auctor nisi eros, 
                        ac dapibus felis mattis eget. Nullam at congue enim.

                        Mauris finibus, mauris id ultricies pellentesque, urna diam aliquet neque, porttitor sagittis
                         tellus ex ac turpis. Suspendisse potenti. Cras laoreet nunc libero. Curabitur eu eros neque. 
                         Ut convallis lacus mollis libero hendrerit congue. Sed ornare, justo a sollicitudin porttitor
                         , dolor ligula ullamcorper elit, ac egestas magna ipsum id nunc. Lorem ipsum dolor sit amet,
                          consectetur adipiscing elit. Curabitur tempus ante et libero condimentum, eget viverra urna
                           hendrerit. Ut malesuada tincidunt mi sed pellentesque. Proin vitae consectetur nisi.
                        
                        Sed at est vel enim eleifend porttitor. Suspendisse quis metus consectetur, pellentesque 
                        sem at, consectetur dui. Aenean sed pellentesque turpis. Donec ut facilisis leo. Maecenas 
                        sed orci ullamcorper, dictum mi ut, feugiat diam. Proin dignissim turpis sit amet tincidunt 
                        dictum. Cras cursus, tellus eget rutrum pretium, magna erat bibendum sapien, quis maximus 
                        tortor eros ut est. Praesent iaculis ligula est, nec viverra lorem ultricies volutpat. Ut i
                        d lorem urna. Maecenas eleifend sagittis sagittis. Mauris convallis congue lobortis.
                        
                        In eleifend semper nisl ac tincidunt. Ut auctor fermentum cursus. Nam id leo vitae ex tempus
                         molestie eget a enim. Phasellus tincidunt ante orci, vel eleifend lectus aliquet vitae. 
                         Vivamus suscipit pellentesque lacus fermentum dignissim. Quisque semper ex ut porta imperdie
                         t. Nunc vulputate malesuada tortor, a dictum sapien euismod ut. Duis nec elementum lacus, 
                         sed tempor nisi. Vestibulum ut massa a ante aliquet malesuada at ultricies libero. Fusce non
                          pulvinar justo, ac congue lorem. Proin neque tortor, rutrum id volutpat non, ullamcorper 
                          fermentum enim. Donec a tortor non est consectetur efficitur at vitae velit. Sed tincidunt
                           magna ullamcorper, tincidunt dui a, venenatis arcu.
                        
                        In vel viverra magna, vitae pellentesque mauris. Nam suscipit, magna vel posuere imperdiet, 
                        arcu nulla hendrerit tortor, a condimentum mauris lacus vel erat. Nunc scelerisque nisi 
                        turpis, a fringilla leo mattis nec. Suspendisse quis facilisis ipsum. Nulla vitae tincidunt 
                        magna. Mauris tempus scelerisque libero sed pulvinar. Vivamus id nulla ut urna congue blandit.
                         Fusce non vehicula eros, in volutpat nunc.
                        """)
